@@ -175,7 +175,7 @@ void sys_setchsr(int chin, int chout, int sr)
 
     /* set audio device settings (after cleaning up the specified device and
     channel vectors).  The audio devices are "zero based" (i.e. "0" means the
-    first one.)  We can later re-open audio and/or show the settings on a\
+    first one.)  We can later re-open audio and/or show the settings on a
     dialog window. */
 
 void sys_set_audio_settings(int naudioindev, int *audioindev, int nchindev,
@@ -192,10 +192,6 @@ void sys_set_audio_settings(int naudioindev, int *audioindev, int nchindev,
     int indevs = 0, outdevs = 0, canmulti = 0, cancallback = 0;
     audio_getdevs(indevlist, &indevs, outdevlist, &outdevs, &canmulti,
         &cancallback, MAXNDEV, DEVDESCSIZE);
-    if (sys_externalschedlib)
-    {
-        return;
-    }
 
     if (rate < 1)
         rate = DEFAULTSRATE;
@@ -426,7 +422,9 @@ void sys_reopen_audio( void)
             chindev, naudiooutdev, audiooutdev, naudiooutdev, choutdev, rate);
     else
 #endif
-        post("unknown audio API specified");
+    if (sys_audioapi == API_NONE)
+        ;
+    else post("unknown audio API specified");
     if (outcome)    /* failed */
     {
         audio_state = 0;
